/* meArm library York Hack Space May 2014
 * A simple control library for Phenoptix' meArm
 * Usage:
 *   meArm arm;
 *   arm.begin(1, 10, 9, 6);
 *   arm.openGripper();
 *   arm.gotoPoint(-80, 100, 140);
 *   arm.closeGripper();
 *   arm.gotoPoint(70, 200, 10);
 *   arm.openGripper();
 */
// Modified by Ted Lien June.2015

#include <Arduino.h>
#include "ik.h"
#include "meArm.h"
#include <Servo.h>
bool setup_servo (ServoInfo& svo, const int n_min, const int n_max,
                  const float a_min, const float a_max)
{
    float n_range = n_max - n_min;
    float a_range = a_max - a_min;

    // Must have a non-zero angle range
    if(a_range == 0) return false;

    // Calculate gain and zero
    svo.gain = n_range / a_range;
    svo.zero = n_min - svo.gain * a_min;

    // Set limits
    svo.n_min = n_min;
    svo.n_max = n_max;

    return true;
}

//Full constructor with calibration data
meArm::meArm(int sweepMinBase, int sweepMaxBase, float angleMinBase, float angleMaxBase,
          int sweepMinShoulder, int sweepMaxShoulder, float angleMinShoulder, float angleMaxShoulder,
          int sweepMinElbow, int sweepMaxElbow, float angleMinElbow, float angleMaxElbow,
          int sweepMinGripper, int sweepMaxGripper, float angleMinGripper, float angleMaxGripper) {
  //calroutine();
  setup_servo(_svoBase, sweepMinBase, sweepMaxBase, angleMinBase, angleMaxBase);
  setup_servo(_svoShoulder, sweepMinShoulder, sweepMaxShoulder, angleMinShoulder, angleMaxShoulder);
  setup_servo(_svoElbow, sweepMinElbow, sweepMaxElbow, angleMinElbow, angleMaxElbow);
  setup_servo(_svoGripper, sweepMinGripper, sweepMaxGripper, angleMinGripper, angleMaxGripper);
}

void meArm::begin(int pinBase, int pinShoulder, int pinElbow, int pinGripper) {
  _pinBase = pinBase;
  _pinShoulder = pinShoulder;
  _pinElbow = pinElbow;
  _pinGripper = pinGripper;
  _base.attach(_pinBase);
  _shoulder.attach(_pinShoulder);
  _elbow.attach(_pinElbow);
  _gripper.attach(_pinGripper);

  x_shoulder = 0; 
  y_elbow = 0;
  servo_speed = 3;

  goDirectlyTo(0, 100, 150);
  delay(100);
  openGripper();
}

//Set servos to reach a certain point directly without caring how we get there 
void meArm::goDirectlyTo(float x, float y, float z) {
  float radBase,radShoulder,radElbow;
  if (solve(x, y, z, radBase, radShoulder, radElbow)) {

    int a0 = int(servodefaultangle[0] - radBase*180/pi );
    int a1 = 180 - int(radShoulder*180/pi + servodefaultangle[1]);
    int a2 = int(radElbow*180/pi + servodefaultangle[2]);

    _base.write(a0+int(xx_base));
    _shoulder.write(a1 + int(x_shoulder));
    _elbow.write(a2 + int(y_elbow));

    // for showAdjustmentsToSerial()
    _baseDeltaAngle = a0 - 90;
    _shoulderDeltaAngle = a1 - 90;
    _elbowDeltaAngle = a2 - 135;

    _x = x; _y = y; _z = z;
  }    
}

//Travel smoothly from current point to another point
void meArm::gotoPoint(float x, float y, float z) {
  //Starting points - current pos
  float x0 = _x; 
  float y0 = _y; 
  float z0 = _z;
  float dist = sqrt((x0-x)*(x0-x)+(y0-y)*(y0-y)+(z0-z)*(z0-z));
  int step = servo_speed;
  
  step = step > 10 ? 10 : step;
  step = step <  1 ?  1 : step;

  for (int i = 0; i<dist; i+= step) {
    goDirectlyTo(x0 + (x-x0)*i/dist, y0 + (y-y0) * i/dist, z0 + (z-z0) * i/dist);
    delay(10);
  }
  //goDirectlyTo(x, y, z);
  delay(50);
}

//Check to see if possible
bool meArm::isReachable(float x, float y, float z) {
  float radBase,radShoulder,radElbow;
  return (solve(x, y, z, radBase, radShoulder, radElbow));
}

//Grab something
void meArm::openGripper() {
  _gripper.write(30);
  delay(100);
}

//Let go of something
void meArm::closeGripper() {
  _gripper.write(0);
  delay(100);
}

//Adjust angles of base/shoulder/elbow servo for the sake of installation difference
void meArm::adjustAngles(float x, float y, float z){
  xx_base = x;
  x_shoulder = y; 
  y_elbow = z;
  
}

//Set speed of moving
//i : 1~9, slowest/fastest
void meArm::setSpeed(int i){
  servo_speed = i;
}

//Show 2 angles for adjustAngles(float x, float y);
void meArm::showAdjustmentsToSerial(){
  Serial.print("adjustAngles(");
  Serial.print(_baseDeltaAngle);
  Serial.print(" , ");
  Serial.print(_shoulderDeltaAngle);
  Serial.print(" , ");
  Serial.print(_elbowDeltaAngle);
  Serial.println(");");

}

//Current x, y and z
float meArm::getX() {
  return _x;
}
float meArm::getY() {
  return _y;
}
float meArm::getZ() {
  return _z;
}